// src/index.ts
import { createRequire } from "module";
import path3 from "path";
import { cac } from "cac";
import { build, dev, serve } from "@rspress/core";
import { logger as logger3 } from "@rspress/shared/logger";
import chokidar from "chokidar";
import chalk from "chalk";

// src/config/loadConfigFile.ts
import fs from "fs";
import path from "path";
import { logger } from "@rspress/shared/logger";

// src/constants.ts
var DEFAULT_CONFIG_NAME = "rspress.config";
var DEFAULT_EXTENSIONS = [
  ".js",
  ".ts",
  ".mjs",
  ".mts",
  ".cjs",
  ".cts"
];

// src/config/loadConfigFile.ts
var findConfig = (basePath) => {
  return DEFAULT_EXTENSIONS.map((ext) => basePath + ext).find(fs.existsSync);
};
async function loadConfigFile(customConfigFile) {
  const baseDir = process.cwd();
  let configFilePath = "";
  if (customConfigFile) {
    if (path.isAbsolute(customConfigFile)) {
      configFilePath = customConfigFile;
    } else {
      configFilePath = path.join(baseDir, customConfigFile);
    }
  } else {
    configFilePath = findConfig(path.join(baseDir, DEFAULT_CONFIG_NAME));
  }
  if (!configFilePath) {
    logger.info(`No config file found in ${baseDir}`);
    return {};
  }
  const { loadConfig } = await import("@rsbuild/core");
  const { content } = await loadConfig({
    cwd: path.dirname(configFilePath),
    path: configFilePath
  });
  return content;
}

// src/update.ts
import path2 from "path";
import fs2 from "fs/promises";
import { spawn } from "child_process";
import { pathExists } from "@rspress/shared/fs-extra";
import { logger as logger2 } from "@rspress/shared/logger";
var lockfileMap = {
  "package-lock.json": "npm",
  "yarn.lock": "yarn",
  "pnpm-lock.yaml": "pnpm",
  "bun.lockb": "bun"
};
async function getPackageManager(rootPath) {
  let packageManager = "npm";
  for (const file of Object.keys(lockfileMap)) {
    if (await pathExists(path2.join(rootPath, file))) {
      packageManager = lockfileMap[file];
      break;
    }
  }
  return packageManager;
}
async function getRspressDependencies(rootPath) {
  const packageJson2 = JSON.parse(
    await fs2.readFile(path2.join(rootPath, "package.json"), {
      encoding: "utf-8"
    })
  );
  const dependencies = packageJson2?.dependencies ? Object.keys(packageJson2.dependencies) : [];
  const devDependencies = packageJson2?.devDependencies ? Object.keys(packageJson2.devDependencies) : [];
  return dependencies.concat(devDependencies).filter((item) => item.startsWith("@rspress") || item.startsWith("rspress"));
}
async function update() {
  const cwd = process.cwd();
  const packageManager = await getPackageManager(cwd);
  const rspressDependencies = await getRspressDependencies(cwd);
  logger2.greet(
    `Using ${packageManager} to update ${rspressDependencies.join(" ")}`
  );
  if (packageManager === "npm" || packageManager === "bun") {
    const dependencies = rspressDependencies.map((item) => `${item}@latest`);
    spawn(packageManager, ["install", "--save", ...dependencies], {
      stdio: "inherit"
    });
  } else {
    const subCommand = packageManager === "yarn" ? "upgrade" : "update";
    spawn(packageManager, [subCommand, "--latest", ...rspressDependencies], {
      stdio: "inherit"
    });
  }
}

// src/index.ts
var CONFIG_FILES = [
  "rspress.config.ts",
  "rspress.config.js",
  "_meta.json",
  "i18n.json"
];
var require2 = createRequire(import.meta.url);
var packageJson = require2("../package.json");
var cli = cac("rspress").version(packageJson.version).help();
var landingMessage = `🔥 Rspress v${packageJson.version}
`;
logger3.greet(landingMessage);
var setNodeEnv = (env) => {
  process.env.NODE_ENV = env;
};
cli.option("-c,--config [config]", "Specify the path to the config file");
cli.command("[root]", "start dev server").alias("dev").option("--port [port]", "port number").option("--host [host]", "hostname").action(
  async (root, options) => {
    setNodeEnv("development");
    let isRestarting = false;
    const cwd = process.cwd();
    let docDirectory;
    let cliWatcher;
    let devServer;
    const startDevServer = async () => {
      const { port, host } = options || {};
      const config = await loadConfigFile(options?.config);
      if (root) {
        config.root = path3.join(cwd, root);
      } else if (config.root && !path3.isAbsolute(config.root)) {
        config.root = path3.join(cwd, config.root);
      }
      docDirectory = config.root || path3.join(cwd, root ?? "docs");
      devServer = await dev({
        appDirectory: cwd,
        docDirectory,
        config,
        extraBuilderConfig: { server: { port, host } }
      });
      cliWatcher = chokidar.watch(
        [`${cwd}/**/{${CONFIG_FILES.join(",")}}`, docDirectory],
        {
          ignoreInitial: true,
          ignored: ["**/node_modules/**", "**/.git/**", "**/.DS_Store/**"]
        }
      );
      cliWatcher.on("all", async (eventName, filepath) => {
        if (eventName === "add" || eventName === "unlink" || eventName === "change" && CONFIG_FILES.includes(path3.basename(filepath))) {
          if (isRestarting) {
            return;
          }
          isRestarting = true;
          console.log(
            `
✨ ${eventName} ${chalk.green(
              path3.relative(cwd, filepath)
            )}, dev server will restart...
`
          );
          await devServer.close();
          await cliWatcher.close();
          await startDevServer();
          isRestarting = false;
        }
      });
    };
    await startDevServer();
    const exitProcess = async () => {
      await cliWatcher.close();
      await devServer.close();
    };
    process.on("SIGINT", exitProcess);
    process.on("SIGTERM", exitProcess);
  }
);
cli.command("build [root]").action(async (root, options) => {
  setNodeEnv("production");
  const cwd = process.cwd();
  const config = await loadConfigFile(options.config);
  if (root) {
    config.root = path3.join(cwd, root);
  }
  await build({
    appDirectory: cwd,
    docDirectory: config.root || path3.join(cwd, root ?? "docs"),
    config
  });
});
cli.command("preview").alias("serve").option("--port [port]", "port number").option("--host [host]", "hostname").action(
  async (options) => {
    setNodeEnv("production");
    const { port, host } = options || {};
    const config = await loadConfigFile(options?.config);
    await serve({
      config,
      host,
      port
    });
  }
);
cli.command("update", "update relevant packages about rspress").action(update);
cli.parse();
